/* Placed in the public domain 2001 by Sam Trenholme */

#include "rijndael/rijndael-api-fst.h"
#include <stdio.h>

int dice_counter = 0;

BYTE r_inBlock[17],r_outBlock[17],r_binKey[17];
BYTE r_keyMaterial[320]; /* We may not eventually need this */
keyInstance r_keyInst;
cipherInstance r_cipherInst;
int r_place = 0;
int dies[3] = { -1, -1, -1 };
int dice_place = 0;

/* A cryptgraphically secure random number with a value between
   0 and 255 (2 to three dice)
   Input: None
   Output: A number between 0 and 255
*/
int dice() {
    int nummake;

    /* If needed, rerun the encryption to create 128 more random bits */
    if(r_place >= 16) {
        r_place = 15;
        nummake = ++dice_counter;
	/* Make sure the dice counter can not step on the seed */
	nummake %= 100000000;
	while(r_place > 0 && nummake > 0) {
            r_inBlock[r_place] = '0' + nummake % 10;
	    nummake /= 10;
	    r_place--;
	    }
#ifdef DEBUG
        printf("\nNEW BLOCK ");
        for(r_place = 0; r_place < 16; r_place++) {
	    printf("%c",r_inBlock[r_place]);
	    }
        printf("\n");
#endif /* DEBUG */
        blockEncrypt(&r_cipherInst,&r_keyInst,r_inBlock,128,r_outBlock);
#ifdef DEBUG
        for(r_place = 0; r_place < 16; r_place++) {
	    printf("%2x ",r_outBlock[r_place]);
	    }
        printf("END NEW BLOCK\n");
#endif /* DEBUG */
	r_place = 0;
	}
    nummake = (r_outBlock[r_place] & 0xff);
    r_place++;
    return nummake;
    }

/* Initialize the cryptographically secure psudo-random-number generator */
/* Input: pointer to string that has binary crypto key */

int init_crypto(char *key) {
    unsigned char crypto_key[34];
    int desc;
    /* Initialize the keys, including the "binKey" (is this used?) */
    memset(r_binKey,'0',16);
    memset(crypto_key,'0',33);
    strncpy(crypto_key,key,32);

    if(makeKey(&r_keyInst, DIR_ENCRYPT, 128, crypto_key) != 1)
	return -1;
    if(cipherInit(&r_cipherInst, MODE_ECB, NULL) != 1)
	return -1;
    if(blockEncrypt(&r_cipherInst,&r_keyInst,r_inBlock,128,r_outBlock) != 128)
	return -1;
    
    return 1;
    }

/* Process the dice rolls.  Input: int from 0 to 255.  Output: None (stdout
   used to show dice rolls) */

void proc_rolls(int rolls) {
    int counter;
    if(rolls < 216) /* 6^3 */ {
        for(counter = 0;counter < 3; counter++) { 
	    printf("%d ",(rolls % 6) + 1);
	    rolls /= 6;
	    }
        printf("\n");
        }
    else if(rolls < 252) {
        rolls -= 216;
        for(counter = 0;counter < 2; counter++) { 
	    printf("%d ",(rolls % 6) + 1);
	    rolls /= 6;
	    }
        printf("\n");
        }
    else {
        printf("<none>\n");
	}
    }

/* Roll a single 6-sided die */

int d6() {
    int counter, rolls;
    /* get more dice rolls, if needed */
    if(dice_place < 0 || dice_place >= 3 || dies[dice_place] == -1) {
        rolls = 255;
	while(rolls >= 252)
            rolls = dice();
        if(rolls < 216) /* 6^3 */ {
            for(counter = 0;counter < 3; counter++) { 
	        dies[counter] = (rolls % 6) + 1;
	        rolls /= 6;
	        }
             }
        else if(rolls < 252) {
            rolls -= 216;
            for(counter = 0;counter < 2; counter++) { 
	        dies[counter] = (rolls % 6) + 1;
	        rolls /= 6;
	        }
             dies[2] = -1;
             }
        else {
            return -1; /* Should never happen */
	    }
        dice_place = 0;
	}
    rolls = dies[dice_place];
    dice_place++;
    return rolls;
    }

int init_rng(seed) {

    int counter;

    /* Set the plaintext block to '0000000000000000' */
    memset(r_inBlock,'0',16);

    /* Make sure the seed can not step on the dice rolls */
    seed %= 100000000;

    /* Put the seed in the block which we encrypt in counter mode (A seed of
       1 makes the plaintext clock look like '1000000000000000', a seed of 256
       makes the plaintext look like '6520000000000000', and so on */
    counter = 0;
    while(seed > 0 && counter < 16) {
        r_inBlock[counter] = '0' + seed % 10;
	seed /= 10;
	counter++;
	}

    /* Initialize the crypto with a fixed key and the above seed */
    init_crypto("1970012508675309");

    }

int main() {
    int roll, counter, input;
    int total;
    int num_to_roll, sign, modifier;
    int seed;
#ifdef DEBUG
    int distro[7];
#endif

    /* Set up a seed for the RNG (a number which reads left to right) */
    seed = 0;
    /* Get the seed from the keyboard */
    printf("Seed for RNG: ");
    input = 0;
    while(input != '\n') {
        input = getc(stdin);
        if(input >= '0' && input <= '9') {
	    seed *= 10;
	    seed += input - '0';
	    }
        } 

    /* Initialize the rng with the above seed */
    init_rng(seed);

    for(;;) {
        printf("Number of dice to roll: ");
        input = 0;
	num_to_roll = 0;
	sign = 1;
	modifier = 0;
	/* Get the number of dice to roll */
        while(input != '\n' && input != '-' && input != '+') {
            input = getc(stdin);
	    if(input >= '0' && input <= '9') {
	        num_to_roll *= 10;
	        num_to_roll += input - '0';
		}
	    if(input == '-')
	        sign = -1;
	    if(input == '+')
	        sign = 1;
            } 
        /* Get the modifier and sign, if appropriate */
        while(input != '\n') {
	    input = getc(stdin);
            if(input >= '0' && input <= '9') {
	        modifier *= 10;
		modifier += input - '0';
		}
            }
        /* Apply the sign to the modifier */
        modifier = modifier * sign;

        /* Start rolling the dice */
        total = 0;
	if(num_to_roll == 0) {
	    continue;
	    }
#ifdef DEBUG
        for(total = 1;total <=6; total++) {
	    distro[total] = 0;
	    }
#endif
	printf("Rolls: ");
	while(num_to_roll > 0) {
            roll = d6();
#ifdef DEBUG
            distro[roll]++;
#endif
            if(num_to_roll < 1000)
	        printf("%d ",roll);
	    total += roll;
	    num_to_roll--;
	    }
        if(modifier != 0) {
	    printf("modifier %d ",modifier);
	    total += modifier;
	    }
        printf("Total %d\n",total);
#ifdef DEBUG
        for(total = 1;total <=6; total++) {
	    printf("Roll %d occured %d times\n",total,distro[total]);
	    }
#endif
	}
    }


